---
title: Documentation Do(d)geBall
---

# Documentation Do(d)geBall

## Table of Contents

[TOC]

## Controls

The game is played locally on one keyboard (with Numpad) and two mice. The following keys' names are derived from a German keyboard layout.

### Player 1

| Key                       | Action         |
| ------------------------- | -------------- |
| W                         | Move Forward   |
| A                         | Move Left      |
| S                         | Move Backwards |
| D                         | Move Right     |
| Left Shift                | Sprint         |
| Space                     | Jump           |
| Mouse 1 Left Mouse Button | Throw Frisbee  |

### Player 2

| Key                       | Action         |
| ------------------------- | -------------- |
| Pos1                      | Move Forward   |
| Entf                      | Move Left      |
| Ende                      | Move Backwards |
| Bild ↓                    | Move Right     |
| Right Shift               | Sprint         |
| Numpad 0                  | Jump           |
| Mouse 2 Left Mouse Button | Throw Frisbee  |

### Miscellaneous

| Key  | Action                  |
| ---- | ----------------------- |
| Esc  | Exit Game               |
| F1   | Toggle Wireframe        |
| F2   | Toggle Backface Culling |

## Technology

### GameObject

GameObjects are objects which exist in a [Scene](#Scene). They can hold multiple child GameObjects and [Components](#Component). Furthermore, they have a transformation (consisting of a translation and a Quaternion rotation), which sets their place in the world space.

GameObjects contain methods which are called during the game loop:

| Method                 | Description                                                  |
| ---------------------- | ------------------------------------------------------------ |
| `triggerUpdate()`      | Gets called every frame, before a potential Physics Simulation Step. (Unlimited times per second) |
| `triggerFixedUpdate()` | Gets called after a Physics Simulation Step. (60 times per second) |
| `triggerDraw()`        | Gets called every frame, after a potential Physics Simulation Step. Indicates that this GameObject is to be drawn right now. (Unlimited times per second) |

The source file for a GameObject can be found under:

`DogeBall/DogeBall/Engine/Scene/GameObject.cpp`

`DogeBall/DogeBall/Engine/Scene/GameObject.h`

### Component

Components are Plug-In-like Code Snippets, which can be attached to [GameObjects](#GameObject) to alter their behavior. Currently there are 5 Components implemented:

| Component                 | Description                                                  |
| ------------------------- | ------------------------------------------------------------ |
| CameraComponent           | Implements a FirstPersonCamera which is attached to the parent [GameObject](#GameObject). |
| ModelComponent            | Holds a [Model](#Model) which follows the transformation of the parent [GameObject](#GameObject) and which is drawn every frame. |
| PlayerComponent           | Implements a PhysX Capsule Controller which lets a player control the parent [GameObject](#GameObject). |
| StaticRigidBodyComponent  | Works only with a ModelComponent! Defines a static Rigid Body in the PhysX engine using the Meshes stored in the ModelComponent. Used for enabling Collisions with the Environment/Map. |
| DynamicRigidBodyComponent | Defines a dynamic Rigid Body in the PhysX engine using the PhysX Shapes added to it. Used for [GameObjects](#GameObject) which are moving in the Scene e.g.: Projectiles. |

Components can implement behaviors using the following methods:

| Method            | Description                                                  |
| ----------------- | ------------------------------------------------------------ |
| `onAwake()`       | Is called once right after the Component has been added to the parent [GameObject](#GameObject). |
| `onStart()`       | Is called once on the first frame this Component exists.     |
| `onEnable()`      | Is called every time the Component is enabled. It is also called on the first frame right after `onStart()` if everything succeeded in `onStart()`. |
| `onUpdate()`      | Is called every frame the Component is active **before** a potential Physics Simulation Step. |
| `onFixedUpdate()` | Is called in regular fixed intervals (60 times per second) if the Component is active, right **after** a Physics Simulation Step. |
| `onDraw()`        | Is called every frame the Component is active **after** a potential Physics Simulation Step. Indicates that the parent [GameObject](#GameObject) is to be drawn right now. |
| `onDisable()`     | Is called every time the Component is disabled. It is also called on the first frame right after `onStart()` if anything fails in `onStart()`. Is **NOT** called when the Component is being deleted from the parent [GameObject](#GameObject) or the parent [GameObject](#GameObject) is being deleted. |
| `onDestroy()`     | Is called once when either the Component is being deleted from the parent [GameObject](#GameObject) or the parent [GameObject](#GameObject) is being deleted. |

### Scene

### Model

## Used Libraries

| Name                     | License                                                      | URL                                      |
| ------------------------ | ------------------------------------------------------------ | ---------------------------------------- |
| Assimp                   | BSD                                                          | https://www.assimp.org/                  |
| GLEW                     | BSD                                                          | http://glew.sourceforge.net/             |
| GLFW                     | [zlib/libpng](https://www.opensource.org/licenses/zlib-license.php) (BSD-like) | https://www.glfw.org/                    |
| OpenGL Mathematics (glm) | The Happy Bunny License (Modified MIT License)               | https://github.com/g-truc/glm            |
| INIReader                | BSD                                                          | https://github.com/benhoyt/inih          |
| NVIDIA PhysX SDK         | BSD                                                          | https://github.com/NVIDIAGameWorks/PhysX |
| spdlog                   | MIT                                                          | https://github.com/gabime/spdlog         |



